// -------------------------------------------------------------------
// Program:   Java Client for Catalog Services Certification JDK 1.2.2
// -------------------------------------------------------------------
// Authors:   Laura Diaz, University of Valencia, Spain
//            Bart v.d. Eijnden, Geodan SDT b.v.
//            Barend Gehrels (BSG), Geodan SDT b.v.
// Copyright: OpenGIS Consortium, inc., all rights reserved worldwide
// Purpose:   Testsuite for conformance of servers claiming compliancy
//            to the OpenGIS Catalog Implementation Specification
// Document:  See also OGC Document 00-027r1 for description
// Date:      may 11, 2000
// -------------------------------------------------------------------
// Revisions: may 27, 2000: (Bart) added query set from file
//            june 9, 2000: (BSG) added query language in file
//                          (BSG) added comments
// -------------------------------------------------------------------

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.*;
import java.lang.*;
import java.io.*;
import org.omg.CORBA.ORB;


// -------------------------------------------------------------------
// Class:     catalogCertification
// -------------------------------------------------------------------
// Purpose:   Main class, containing certificaton suite
// -------------------------------------------------------------------
public class catalogCertification extends JFrame
{
	public Vector QueryVector = new Vector();
	public Vector QueryLanguageVector = new Vector();
	public Vector NumberOfHits = new Vector();


	public class c_capabilities
	{
		private String[] name;
		private int[] available;
		//available = 0 -> not supported
		//available = 1 -> supported
		//available = 2 -> not tested		
		public c_capabilities()
		{
			try
			{
				InputStream in = new BufferedInputStream(new FileInputStream("query.txt"));
				BufferedReader dis = new BufferedReader(new InputStreamReader(in));
				String line = null;
				while ( (line = dis.readLine()) != null)
				{
					StringTokenizer split = new StringTokenizer(line,"	");
					NumberOfHits.addElement(split.nextToken());
					QueryLanguageVector.addElement(split.nextToken());
					QueryVector.addElement(split.nextToken());
				}
				name = new String[(38 + QueryVector.size())];
				available = new int[(38 + QueryVector.size())];		
			}
			catch(Exception e)
			{
				ExceptionReport("getquery", e);
			}
			for (int i=0; i < (38+QueryVector.size());i++)
				available[i] = 2;				
			writeTableCapabilities();
		}

		private void writeTableCapabilities()
		{
			name[0] = "Init Session";
			name[1] = "Explain Collection";
			name[2] = "Explain Collection collectionID = collectionID";
			name[3] = "Explain Collection collectionID = collectionName";			
			name[4] = "Explain Collection Format = XML";
			name[5] = "Explain Collection Format = HTML";
			name[6] = "Explain Collection Format = TXT";
			name[7] = "Explain Collection Format = NV";
			name[8] = "Explain Server";
			name[9] = "Query";
			name[10] ="Query collectionID = collectionID";
			name[11] ="Query collectionID = collectionName";
			name[12] ="Query Expression Language = OGC_Common";
			name[13] ="Query Expression Language = Z3950_TypeOne";
			name[14] ="Query Message Format = XML";
			name[15] ="Query Message Format = HTML";
			name[16] ="Query Message Format = TXT";
			name[17] ="Query Message Format = NV";
			name[18] ="Query number of Hits retrieved";
			name[19] ="Query sortField sortOrder = ascending";
			name[20] ="Query sortField sortOrder = descending";
			name[21] ="Query result Type = results";
			name[22] ="Query result Type = validate";
			name[23] ="Query result Type = hits";
			name[24] ="Query result Type = resultSetID";
			name[25] ="Query Scope = local";
			name[26] ="Query Scope = distributed";
			name[27] ="Query GeographicBoundingBox";
			name[28] ="Invalid query request";
			name[29] ="Present";
			name[30] ="Present Message Format = XML";
			name[31] ="Present Message Format = HTML";
			name[32] ="Present Message Format = TXT";
			name[33] ="Present Message Format = NV";
			name[34] ="Present number of Hits retrieved";
			name[35] ="Present sortField sortOrder = ascending";
			name[36] ="Present sortField sortOrder = descending";
			name[37] ="Terminate Session";
			for (int i=0; i<(QueryVector.size()); ++i)
			{
				name[38+i] = "Query " + QueryVector.elementAt(i);
			}
		}
	}
		
	
	// CORBA support
	private org.omg.CORBA.ORB m_Orb;
	private OGC_CatalogService.CG_CatalogServices m_Catalog = null;

	// Program logic
	private int m_SessionID = 0;
	private int m_Counter = 0;
	private Vector m_OrderVector = new Vector(); // Global variable for storing ordered attribute 
        private c_capabilities tableCapabilities = new c_capabilities();
	static private int correctHits = 3; // number of Hits I know is correct
	static private int correctHitsCoord = 7; //number of Hits I know is correct in boundingbox query()
	
	
	//-------------- DECLARE_CONTROLS ----------------

	private JPanel queryPanel = new JPanel();
	private JPanel capabilitiesPanel = new JPanel();
	private JScrollPane capabilitieScroll = new JScrollPane();
	private JScrollPane queryScroll = new JScrollPane();

	private GridBagLayout theGridBagCap = new GridBagLayout();
	private GridBagConstraints theConstraintsCap = new GridBagConstraints();
	
	private ButtonGroup languageButtons = new ButtonGroup();
	private JRadioButton language1 = new JRadioButton();
	private JRadioButton language2 = new JRadioButton();
	private JRadioButton language3 = new JRadioButton();
	
	private JScrollPane resultsAreaPane = new JScrollPane();
	private JScrollPane diagnosticAreaPane = new JScrollPane();
	private JTextField queryText = new JTextField();
	static JButton submitButton = new JButton();
	private JTextArea resultsArea = new JTextArea(5,41);
	private JTextArea diagnosticArea = new JTextArea(1,41);
	private JTextField statusText = new JTextField();

	private JLabel languageLabel = new JLabel();
	private JLabel queryLabel = new JLabel();
	private JLabel resultsLabel = new JLabel();
	private JLabel diagnosticLabel = new JLabel();
	private JLabel capabilitiesLabel = new JLabel();
	private JLabel currentStatusLabel = new JLabel();
	
	//------------------------------------------------------------------------
		
public void drawQueryPart()
{
		queryScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		queryScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);		
		getContentPane().add(queryScroll);
		queryScroll.getViewport().add(queryPanel);
		GridBagLayout theGridBag = new GridBagLayout();
		queryPanel.setLayout(theGridBag);
		GridBagConstraints theConstraints = new GridBagConstraints();				
		queryPanel.setBackground(new java.awt.Color(204,204,204));
		theConstraints.fill = GridBagConstraints.BOTH; 
		theConstraints.gridwidth = GridBagConstraints.REMAINDER;
		theConstraints.anchor = GridBagConstraints.WEST;
		theConstraints.weightx = 1.0;
		languageLabel.setText("Language");
		theGridBag.setConstraints(languageLabel,theConstraints);
		queryPanel.add(languageLabel);			
		language1.setText("OGC_Common");		
		language1.setSelected(true);
		theConstraints.ipady = 5;
		theGridBag.setConstraints(language1,theConstraints);
		queryPanel.add(language1);
		language2.setText("Z3950 Type One");
		theGridBag.setConstraints(language2,theConstraints);
		queryPanel.add(language2);
		language3.setText("SQL 3 Simple Features");
		theGridBag.setConstraints(language3,theConstraints);
		queryPanel.add(language3);
		languageButtons.add(language1);
		languageButtons.add(language2);
		languageButtons.add(language3);		
		queryLabel.setText("Query");
		queryLabel.setAutoscrolls(true);
		theGridBag.setConstraints(queryLabel,theConstraints);
		queryPanel.add(queryLabel);
		theConstraints.gridwidth = GridBagConstraints.RELATIVE;
		queryText.setText("title like '%Europe%'");
		//queryText.setText("title like '%Asia%'");
		theGridBag.setConstraints(queryText,theConstraints);
		queryPanel.add(queryText);
		submitButton.setText("Submit");
		submitButton.setActionCommand("Submit");
		submitButton.setAutoscrolls(true);
		submitButton.addActionListener(new submitQuery());

		theConstraints.gridwidth = GridBagConstraints.REMAINDER;
		theConstraints.weightx=0.0; 
		theGridBag.setConstraints(submitButton,theConstraints);
		queryPanel.add(submitButton);
		resultsLabel.setText("Results");
		resultsLabel.setAutoscrolls(true);
		theConstraints.ipady = 10;
		theGridBag.setConstraints(resultsLabel,theConstraints);
		queryPanel.add(resultsLabel);
		resultsAreaPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		resultsAreaPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);		
		theConstraints.ipady = 250;
		theConstraints.fill = GridBagConstraints.NONE;
		theConstraints.anchor = GridBagConstraints.CENTER;
		theGridBag.setConstraints(resultsAreaPane,theConstraints);
		queryPanel.add(resultsAreaPane);		
		resultsAreaPane.getViewport().add(resultsArea);
		resultsArea.setEditable(false);		
		diagnosticLabel.setText("Diagnostic");
		theConstraints.anchor = GridBagConstraints.WEST;
		theConstraints.ipady = 10;
		theConstraints.fill = GridBagConstraints.BOTH;
		theGridBag.setConstraints(diagnosticLabel,theConstraints);
		queryPanel.add(diagnosticLabel);
		diagnosticAreaPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		diagnosticAreaPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);		
		theConstraints.ipady = 65;
		theConstraints.fill = GridBagConstraints.NONE;
		theConstraints.anchor = GridBagConstraints.CENTER;
		theGridBag.setConstraints(diagnosticAreaPane,theConstraints);
		queryPanel.add(diagnosticAreaPane);
		diagnosticAreaPane.getViewport().add(diagnosticArea);
		diagnosticArea.setEditable(false);				

		submitButton.setEnabled(false);

		theConstraints.anchor = GridBagConstraints.NORTHWEST;
		theConstraints.gridwidth = GridBagConstraints.REMAINDER;
		theConstraints.ipady = 10;
		currentStatusLabel.setText("Current Status");
		theGridBag.setConstraints(currentStatusLabel,theConstraints);
		queryPanel.add(currentStatusLabel);	
		theConstraints.ipady = 5;
		theConstraints.ipadx = 200;
		theGridBag.setConstraints(statusText,theConstraints);
		queryPanel.add(statusText);
		statusText.setEditable(false);		
}

public void drawCapabilitiesPart()
{
		capabilitieScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		capabilitieScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);		
		getContentPane().add(capabilitieScroll);
		capabilitieScroll.getViewport().add(capabilitiesPanel);
		capabilitiesPanel.setLayout(theGridBagCap);	

		capabilitiesPanel.setBackground(new java.awt.Color(204,204,204));
		theConstraintsCap.fill = GridBagConstraints.HORIZONTAL;
		theConstraintsCap.gridwidth = GridBagConstraints.REMAINDER;
		theConstraintsCap.weightx = 1.0;
		theConstraintsCap.ipady = 15;

		JLabel theLab = new JLabel("Capabilities");
		theGridBagCap.setConstraints(theLab,theConstraintsCap);
		capabilitiesPanel.add(theLab);		
}

public void addCapability(String s, int supported)
{	
		Image yes = (Toolkit.getDefaultToolkit()).getImage("images/checkit.gif");
		Image no = (Toolkit.getDefaultToolkit()).getImage("images/no.gif");
		Image cuestion = (Toolkit.getDefaultToolkit()).getImage("images/cuestion.gif");
		ImageIcon cuestionIcon = new ImageIcon(cuestion); 
		ImageIcon yesIcon = new ImageIcon(yes); 
		ImageIcon noIcon = new ImageIcon(no); 

		theConstraintsCap.ipady = 5;
		theConstraintsCap.gridwidth = GridBagConstraints.REMAINDER;

		if (supported == 1)
		{
			JLabel theLab = new JLabel(s,yesIcon,JLabel.LEFT);
			theLab.setVerticalTextPosition(JLabel.CENTER);
			theLab.setHorizontalTextPosition(JLabel.LEFT);
			theLab.setForeground(java.awt.Color.black);		
			theGridBagCap.setConstraints(theLab,theConstraintsCap);
			capabilitiesPanel.add(theLab);
			capabilitiesPanel.updateUI();
		}
		else
		{
			if (supported == 0)
			{			
				JLabel theLab = new JLabel(s,noIcon,JLabel.LEFT);
				theLab.setVerticalTextPosition(JLabel.CENTER);
				theLab.setHorizontalTextPosition(JLabel.LEFT);
				theLab.setForeground(java.awt.Color.black);
				theGridBagCap.setConstraints(theLab,theConstraintsCap);
				capabilitiesPanel.add(theLab);
				capabilitiesPanel.updateUI();
			}
			else
			{				
				JLabel theLab = new JLabel(s,cuestionIcon,JLabel.LEFT);
				theLab.setVerticalTextPosition(JLabel.CENTER);
				theLab.setHorizontalTextPosition(JLabel.LEFT);
				theLab.setForeground(java.awt.Color.black);		
				theGridBagCap.setConstraints(theLab,theConstraintsCap);
				capabilitiesPanel.add(theLab);
				capabilitiesPanel.updateUI();
			}
		}		
}

public catalogCertification()  // Class Client Constructor
{		
		setTitle("OpenGis Catalog Certification");
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		getContentPane().setLayout(new GridLayout(1,1,0,0));
		setSize(970,695);
		setVisible(false);

		drawQueryPart();
		drawCapabilitiesPart();	
	
		// --- REGISTER_LISTENERS ---
		SymWindow aSymWindow = new SymWindow();
		this.addWindowListener(aSymWindow);
}


// -------------------------------------------------------------------
// Method:    catalogCertification, constructor
// -------------------------------------------------------------------
// Purpose:   Creates a new instance of Client with the given title. 
// -------------------------------------------------------------------
public catalogCertification(String sTitle)
{
	this();
	setTitle(sTitle);
}


// -------------------------------------------------------------------
// Method:    setStatusCapability
// -------------------------------------------------------------------
// Purpose:   fill the tablecapabilities with the correct status in each position
// -------------------------------------------------------------------
private void setStatusCapability(int pos, int status)
{
	tableCapabilities.available[pos] = status;
}


// -------------------------------------------------------------------
// Method:    getStatusCapability
// -------------------------------------------------------------------
// Purpose:   returns the state of the  capability in "pos" position
// -------------------------------------------------------------------
private int getStatusCapability(int pos)
{
	return(tableCapabilities.available[pos]);
}

// -------------------------------------------------------------------
// Method:    baseGet
// -------------------------------------------------------------------
// Purpose:   fills out the base of each message
// -------------------------------------------------------------------
private OGC_CatalogService.CG_Message BaseGet()
{
	return new OGC_CatalogService.CG_Message(m_SessionID,"",
			new OGC_CatalogService.CG_RequestID(m_SessionID, m_Counter++),"");
}


// -------------------------------------------------------------------
// Method:    ExceptionReport
// -------------------------------------------------------------------
// Purpose:   Show the errors when a exception occurs
// -------------------------------------------------------------------
private void ExceptionReport(String s, Exception e)
{
		diagnosticArea.append("Error in " + s + ": " + e.getMessage()+ "\n");
		e.printStackTrace();
}

// -------------------------------------------------------------------
// Method:    
// -------------------------------------------------------------------
// Purpose:   
// -------------------------------------------------------------------
public boolean initSession(String[] args)
{						
		String theior = "";

		setStatus("Initializing Session ");
		try
		{
			// Open InputStream, then open DataInputStream
			InputStream in = new BufferedInputStream(new FileInputStream("ogcatsrv.ior"));
            		BufferedReader dis = new BufferedReader(new InputStreamReader(in));
			// Read IOR from first line (there is only one line)
			theior = dis.readLine();
		}
		catch(Exception e)
		{								
			ExceptionReport("getior", e);
			// Exception. There is no IOR, we can return because we can not get objects
			return false;
		}
		// Initialize the ORB and catalog object from ior
		try
		{
			m_Orb = org.omg.CORBA.ORB.init(args, null);

			//narrow : used for typecast, it lets you locate server-side objects references of a more derived type
			m_Catalog = OGC_CatalogService.CG_CatalogServicesHelper.narrow(m_Orb.string_to_object(theior));			
		} 
		catch (Exception e)
		{
			// if narrow fails a CORBA::BAD_PARAM is thrown
			ExceptionReport("connect", e);
			return false;
		}
		if (m_Catalog != null)
		{
			OGC_CatalogService.CG_InitSessionRequest r = new OGC_CatalogService.CG_InitSessionRequest();
			r.base = BaseGet();	
			OGC_CatalogService.CG_InitSessionResponse resp;
			try
			{
				resp = m_Catalog.initSession(r); 
				m_SessionID = resp.base.sessionID;				
			} 
			catch (Exception e)
			{
				ExceptionReport("initSession", e);
				return false;
			}
			return true;
		}
		else
			return false; // m_Catalog == null
} 

	
// -------------------------------------------------------------------
// Method:    terminateSession
// -------------------------------------------------------------------
// Purpose:   tests the terminateSession request
// -------------------------------------------------------------------
public boolean terminateSession()
{
		// Session must be terminated to free resources on the serverside
		if (m_Catalog != null && m_SessionID != 0)
		{
			OGC_CatalogService.CG_TerminateRequest r= new OGC_CatalogService.CG_TerminateRequest();
			r.base = BaseGet();

			OGC_CatalogService.CG_TerminateResponse resp;
			try
			{
				setStatus("Terminating Session");
				resp = m_Catalog.terminateSession(r);

				
				if(resp.status == OGC_CatalogService.CG_Status.success)
				{
					diagnosticArea.append("Terminate Session Status = success \n");
					return(true);
				}
				else
				{
					if(resp.status == OGC_CatalogService.CG_Status.successresultsAvailable)
					{
						diagnosticArea.append("Terminate Session Status = successresultsAvailable \n");
						return(true);					
					}
					else
						if(resp.status == OGC_CatalogService.CG_Status.processingNormal)
						{
							diagnosticArea.append("Terminate Session Status = processingNormal \n");
							return(false);
						}
						else
						{
							if (resp.status == OGC_CatalogService.CG_Status.processingpausedOrSuspended)
							{
								diagnosticArea.append("Terminate Session Status = processingpausedOrSuspended \n");
								return(false);
							}
							else
							{
								diagnosticArea.append("Terminate Session Status = failure \n");
								return(false);
							}						
						}					
				}
				
			} 
			catch (Exception e)	{
				ExceptionReport("terminateSession" , e);
				return(false);
			}
		}
		else
			return(false);
}


// -------------------------------------------------------------------
// Method:    addString
// -------------------------------------------------------------------
// Purpose:   add element to the Global Vector which server for testing the order
// -------------------------------------------------------------------
private void addString(String name)
{
	m_OrderVector.addElement(name);
}


// -------------------------------------------------------------------
// Method:    showStatus
// -------------------------------------------------------------------
// Purpose:   Show status in statuswindow or diagnostig area
// -------------------------------------------------------------------
private void showStatus(String s,OGC_CatalogService.CG_Status status)
{
	if(status == OGC_CatalogService.CG_Status.success)
		diagnosticArea.append(s +": Status = success \n");			
	else
		if(status == OGC_CatalogService.CG_Status.processingNormal)
			diagnosticArea.append(s +": Status = Processing Normal \n");
		else
			if (status == OGC_CatalogService.CG_Status.processingpausedOrSuspended)
				diagnosticArea.append(s +": Status = Processing Paused or Suspended \n");
			else
				diagnosticArea.append(s +": Status = failure \n");
}


// -------------------------------------------------------------------
// Method:    getAttribute
// -------------------------------------------------------------------
// Purpose:   Obtain the attribute to be ordered for inserting it into the Global Vector
// -------------------------------------------------------------------
private void getAttribute(OGIS.NVPair p, int Level,String Attribute)
{
	try
	{
		if (p.value.type().kind() == org.omg.CORBA.TCKind.tk_string)
			if (p.name.indexOf(Attribute)!=-1)				
				addString(p.value.extract_string());
			else{}
		else
		{
			OGIS.NVPair[] Value = OGIS.NVPairSeqHelper.read(p.value.create_input_stream());
			for (int i = 0; i < Value.length; i++)
			{
				getAttribute(Value[i], Level + 1,Attribute);
			}
		}
	}
	catch(Exception e){
		System.out.println("Ignoring, getAttribute :" + e);
	}
}


// -------------------------------------------------------------------
// Method:    testOrder
// -------------------------------------------------------------------
// Purpose:   Testing if the Order returned is OK
//            the m_OrderVector variable contains the String[] for testing
// -------------------------------------------------------------------
public boolean testOrder(String Order)
{
		boolean correct = true;
		int i;

		if(Order.indexOf("ascending")!= -1)
		{
			for (i=0;i<m_OrderVector.size()-1;i++)
			{
				if(m_OrderVector.elementAt(i).toString().compareTo(m_OrderVector.elementAt(i+1).toString())>0)
					correct = false;
			}
		}
		else // Order=="descending"
		{
			for (i=0;i<m_OrderVector.size()-1;i++)
			{
				if(m_OrderVector.elementAt(i).toString().compareTo(m_OrderVector.elementAt(i+1).toString())<0)
					correct = false;
			}
		}
		/*
		System.out.println("Vector size: " + m_OrderVector.size());
		for (i=0;i<m_OrderVector.size();i++)
			System.out.println("Vector[i] :" + m_OrderVector.elementAt(i).toString());
		*/
		
		return(correct);
}


// -------------------------------------------------------------------
// Method:    NVAdd
// -------------------------------------------------------------------
// Purpose:   Extract results with the NV Format and shows them in the 
//            Area Results. This function calls itself to add subrecords
// -------------------------------------------------------------------
private void NVAdd(OGIS.NVPair p, int Level)
{
		try
		{
			if (p.value.type().kind() == org.omg.CORBA.TCKind.tk_string)
			{
				String Indent = "";
				for (int i = 1; i < Level; i++)
				{
					Indent = Indent + "  ";	
				}
				addResults(Indent + p.name + ": " + p.value.extract_string() + "\n");
			}
			else
			{
				//System.out.println(" p.value.type != org.omg.CORBA.TCKind.tk_string");
				OGIS.NVPair[] Value = OGIS.NVPairSeqHelper.read(p.value.create_input_stream());
				for (int i = 0; i < Value.length; i++)
				{
					NVAdd(Value[i], Level + 1);
				}
			}
		}
		catch(Exception e){
			System.out.println("Ignoring");
		} 
}


// -------------------------------------------------------------------
// Method:    addResults
// -------------------------------------------------------------------
// Purpose:   Write results in the resultsArea
// -------------------------------------------------------------------
private void addResults(String s)
{
		try
		{
			resultsArea.append(s);
		}
		catch(Exception e){
			System.out.println("Ignoring");
		} 
}


// -------------------------------------------------------------------
// Method:    getLanguage
// -------------------------------------------------------------------
// Returns:   query Language selected depending on the radio button selected
// -------------------------------------------------------------------
OGC_CatalogService.CG_QueryLanguage getLanguage()
{
	if (language1.isSelected()== true)
		return(OGC_CatalogService.CG_QueryLanguage.OGC_Common);
	else
		if (language2.isSelected()== true)
			return(OGC_CatalogService.CG_QueryLanguage.Z3950_TypeOne); 
		else
			return(OGC_CatalogService.CG_QueryLanguage.SQL3_SimpleFeature);
}


// -------------------------------------------------------------------
// Method:    isCorrect
// -------------------------------------------------------------------
// Return:    true if status is successfull, else false
// -------------------------------------------------------------------
boolean isCorrect(OGC_CatalogService.CG_Status status)
{
	boolean correct = true;

	if((status == OGC_CatalogService.CG_Status.success)||(status == OGC_CatalogService.CG_Status.successresultsAvailable))
		return(true);		
	else
		return(false);
}


// -------------------------------------------------------------------
// Method:    testFormatQuery
// -------------------------------------------------------------------
// Purpose:   this function shows the supported Formats (XML, HTML, TXT, NV) 
//            in the query function
// -------------------------------------------------------------------
void testFormatQuery(OGC_CatalogService.CG_QueryRequest r, OGC_CatalogService.CG_QueryResponse resp)
{
	try
	{
		if (r.returnFormat == resp.retrievedData.encoding)
		{
			if (resp.retrievedData.encoding == OGC_CatalogService.CG_MessageFormat.XML)
			{
				try
				{																		
					String s = resp.retrievedData.payload.extract_string();									
					// Does the "<?xml" string exits?
					if(s.indexOf("<?xml")!= -1)
						setStatusCapability(14,1);// xml
					else
						setStatusCapability(14,0);// xml
				}
				catch( Exception e ){
					setStatusCapability(14,0);}
			}
			else if (resp.retrievedData.encoding == OGC_CatalogService.CG_MessageFormat.NV)
				{
					try
					{
						OGIS.NVPair[] Meta = OGIS.NVPairSeqHelper.read(resp.retrievedData.payload.create_input_stream());
						// Add all hits
						for (int i = 0; i < Meta.length; i++)
						{
							NVAdd(Meta[i], 0);
							getAttribute(Meta[i], 0,"Title"); //Store title in m_OrderVector for testing the order
						}

						if((Meta.length == correctHits)&&(Meta.length == resp.hits))
						{
							setStatusCapability(18,1);// number of Hits
							setStatusCapability(17,1);// NV
							setStatusCapability(9,1);//query
						}
						else
						{
							if (Meta.length == correctHits)// correct NV and query, wrong number of Hits
							{
								setStatusCapability(18,0);// number of Hits
								setStatusCapability(17,1);// NV
								setStatusCapability(9,1);//query
							}
							else
							{
								if (Meta.length == resp.hits)
								{
									setStatusCapability(18,1);// number of Hits
									setStatusCapability(17,0);// NV
									setStatusCapability(9,0);//query
								}
							}							
						}														
					}
					catch (Exception e){
						setStatusCapability(17,0);}
				}
				else
					if (resp.retrievedData.encoding == OGC_CatalogService.CG_MessageFormat.HTML)
					{
						try
						{																		
							String s = resp.retrievedData.payload.extract_string();							
							// Does the "<html>" string exits?
							if((s.indexOf("<html>")!= -1)&&(s.indexOf("</html>")!= -1))
								setStatusCapability(15,1);// html
							else
								setStatusCapability(15,0);// html
						}
						catch( Exception e ){
							setStatusCapability(15,0);}
					}
					else
						if (resp.retrievedData.encoding == OGC_CatalogService.CG_MessageFormat.TXT)
						{	
							try
							{																		
								String s = resp.retrievedData.payload.extract_string();
								setStatusCapability(16,1);
							}
							catch( Exception e ){
								setStatusCapability(16,0);}
						}
						else
							diagnosticArea.append("Unsupported encoding: Error in data Format (query)\n");				
		}
		else
		{				
			if (r.returnFormat == OGC_CatalogService.CG_MessageFormat.TXT)
			{
				diagnosticArea.append("Query: TXT format not supported \n");	
				setStatusCapability(16,0);
			}
			else
				if (r.returnFormat == OGC_CatalogService.CG_MessageFormat.HTML)
				{
					diagnosticArea.append("Query: HTML format not supported \n");	
					setStatusCapability(15,0);
				}
				else
					if(r.returnFormat == OGC_CatalogService.CG_MessageFormat.XML)
					{
						diagnosticArea.append("Query: XML format not supported \n");	
						setStatusCapability(14,0);
					}
					else// NV
					{
						diagnosticArea.append("Query: NV format not supported \n");	
						setStatusCapability(17,0);
					}
		}
	}
	catch(Exception e){
		ExceptionReport("any", e);
	}
}

public void removeVector()
{
	 m_OrderVector.removeAllElements();
}


// -------------------------------------------------------------------
// Method:    testFormatPresent
// -------------------------------------------------------------------
// Purpose:   Test the status of the formats in the present() function
// -------------------------------------------------------------------
void testFormatPresent(OGC_CatalogService.CG_PresentRequest r, OGC_CatalogService.CG_PresentResponse resp)
{
	try
	{
		if (r.returnFormat == resp.retrievedData.encoding)
		{
			if (resp.retrievedData.encoding == OGC_CatalogService.CG_MessageFormat.XML)
			{				
				try
				{																		
					String s = resp.retrievedData.payload.extract_string();					
					if (s.indexOf("<?xml")!= -1)							
						setStatusCapability(30,1);							
					else
						setStatusCapability(30,0);
				}
				catch( Exception e ){
					setStatusCapability(30,0);}
			}
			else if (resp.retrievedData.encoding == OGC_CatalogService.CG_MessageFormat.NV)
				{
					try
					{						
						OGIS.NVPair[] Meta = OGIS.NVPairSeqHelper.read(resp.retrievedData.payload.create_input_stream());
						
						// Add all hits
						for (int i = 0; i < Meta.length; i++)
						{
							getAttribute(Meta[i], 0,"Title"); // for testing the order
						}

						if((Meta.length == correctHits)&&(Meta.length == resp.hits))
						{
							setStatusCapability(34,1);// number of Hits
							setStatusCapability(33,1);// NV
							setStatusCapability(29,1);//present
						}
						else
						{
							if (Meta.length == correctHits)// correct NV and query, wrong number of Hits
							{
								setStatusCapability(34,0);// number of Hits
								setStatusCapability(33,1);// NV
								setStatusCapability(29,1);//present
							}
							else
							{
								if (Meta.length == resp.hits)
								{
									setStatusCapability(34,1);// number of Hits
									setStatusCapability(33,0);// NV
									setStatusCapability(29,0);//present
								}
							}							
						}					
					}
					catch (Exception e){
						setStatusCapability(33,0);
					}
				}
				else
					if (resp.retrievedData.encoding == OGC_CatalogService.CG_MessageFormat.HTML)
					{
						try
						{																									
							String s = resp.retrievedData.payload.extract_string();
							if ((s.indexOf("<html>")!= -1)&&(s.indexOf("</html>")!= -1))							
								setStatusCapability(31,1);							
							else
								setStatusCapability(31,0);
						}
						catch( Exception e ){
							setStatusCapability(31,0);}
					}
					else
						if (resp.retrievedData.encoding == OGC_CatalogService.CG_MessageFormat.TXT)
						{	
							try
							{																										
								String s = resp.retrievedData.payload.extract_string();
								setStatusCapability(32,1);
							}
							catch( Exception e ){
								setStatusCapability(32,0);}
						}
						else
							diagnosticArea.append("Unsupported encoding: Error in data Format (present) \n");				
		}
		else
		{
			
			if (r.returnFormat == OGC_CatalogService.CG_MessageFormat.TXT)
			{
				diagnosticArea.append("Present: TXT format not supported \n");	
				setStatusCapability(32,0);
			}
			else
				if (r.returnFormat == OGC_CatalogService.CG_MessageFormat.HTML)
				{
					diagnosticArea.append("Present: HTML format not supported \n");	
					setStatusCapability(31,0);
				}
				else
					if(r.returnFormat == OGC_CatalogService.CG_MessageFormat.XML)
					{
						diagnosticArea.append("Present: XML format not supported \n");	
						setStatusCapability(30,0);
					}
					else // NV Format
					{
						diagnosticArea.append("Present: NV format not supported \n");	
						setStatusCapability(33,0);
					}					
		}
	}
	catch(Exception e){
		ExceptionReport("any", e);
	}
}


// -------------------------------------------------------------------
// Method:    noDataReturned
// -------------------------------------------------------------------
// Purpose:   test if retrievedData contains data
// Return:    true  -> if the retrieveData of the Queryresponse contains no Data
//            false -> retrieveData contains data
// -------------------------------------------------------------------
boolean noDataReturned(OGC_CatalogService.CG_QueryResponse resp)
{
	try
	{		
		/*
		OGIS.NVPair[] Meta = OGIS.NVPairSeqHelper.read(resp.retrievedData.payload.create_input_stream());
		if (Meta.length != 0)		
		{
			System.out.println("Meta.length no es 0 hay datos en retrieve data...");
			return(false);	
		}
		else
		{
			System.out.println("Meta.length es 0... ie, no hay datos en retrieve data");
			return(true);
		}
		*/
		
		String s = resp.retrievedData.payload.extract_string();
		
		if (s.compareTo("")==0)
		{
			return(true);
		}
		else
			return(false);

	}
	catch(Exception e)
	{
		System.out.println("Exception en noDataReturned:" + e);
		return (false);
		//return (true) I know the exception is due a null string, so it is empty...?
	}
}

// -------------------------------------------------------------------
// Method:    numberHits
// -------------------------------------------------------------------
// Purpose:   Reads the number of hits from the NV pairs
// Return:    Number of hits in response
// -------------------------------------------------------------------
int numberHits(OGC_CatalogService.CG_QueryResponse resp)
{
	try
	{		
		OGIS.NVPair[] Meta = OGIS.NVPairSeqHelper.read(resp.retrievedData.payload.create_input_stream());
		return(Meta.length);
	}
	catch(Exception e)
	{
		return (-1);
	}
}


// -------------------------------------------------------------------
// Method:    queryTest
// -------------------------------------------------------------------
// Purpose:   it makes the default query testing all the capabilities
//            of the server creating diferents query request...
// -------------------------------------------------------------------
void queryTest(String thequery)
{
		OGC_CatalogService.CG_QueryLanguage language;
		OGC_CatalogService.CG_QueryRequest r= new OGC_CatalogService.CG_QueryRequest();
		r.base = BaseGet();
		language = getLanguage();
		int localHits;
		boolean empty;
		
		r.queryExpression = new OGC_CatalogService.CG_QueryExpression(thequery,"",language);
		r.iteratorSize = 100;
		r.cursor = 0;
		r.asynchronous = false;
		r.maxLevel = 2;
		r.queryScope = OGC_CatalogService.CG_QueryScope.local;		

		r.presentation = new OGC_CatalogService.CG_PresentationDescription();
		r.presentation.presentationType(OGC_CatalogService.CG_PredefinedPresentationType.full);

		r.sortField = new OGC_CatalogService.CG_SortField[0];  	

		r.collectionID = new OGC_CatalogService.CG_CollectionName();	
		r.collectionID.collectionID("");
		r.catalogType = OGC_CatalogService.CG_CatalogEntryType.collection;		
		
		r.resultType = OGC_CatalogService.CG_ResultType.results;
		r.returnFormat = OGC_CatalogService.CG_MessageFormat.XML;
		
		OGC_CatalogService.CG_QueryResponse resp;
		
		setStatus("Making query");			

		try
		{
			resp = m_Catalog.query(r);
			try
			{
				if (isCorrect(resp.status))
				{
					System.out.println("Query resp.resultsetID.collectionID():"+resp.resultSetID.collectionID());							
					setStatusCapability(10,1);
				}
				else
					setStatusCapability(10,0);
			}
			catch(Exception e)
			{
				setStatusCapability(10,0);
			}

			if (isCorrect(resp.status))
			{
				testFormatQuery(r,resp);			
				setStatusCapability(21,1); // results
			}
			else
				setStatusCapability(9,0);
		}
		catch(Exception e)
		{
			ExceptionReport("Testing collectionID in Query ", e);
			setStatusCapability(9,0);
			setStatusCapability(21,0); // results
		}
		
		r.collectionID.collectionName("");
		try
		{
			resp = m_Catalog.query(r);
			try
			{				
				if (isCorrect(resp.status))
				{
					System.out.println("resp.resultsetID.collectionID():"+resp.resultSetID.collectionName());
					setStatusCapability(11,1);
					setStatusCapability(21,1); // results
				}
				else
					setStatusCapability(11,0);
			}
			catch(Exception e)
			{
				setStatusCapability(11,0);
			}						
		}
		catch(Exception e)
		{
			setStatusCapability(21,0);//results
			ExceptionReport("Testing collectionName in Query ", e);
		}
		
		r.collectionID.collectionID("");
		r.returnFormat = OGC_CatalogService.CG_MessageFormat.HTML; 
		try
		{	
			resp = m_Catalog.query(r);
			if(isCorrect(resp.status))
			{
				testFormatQuery(r,resp);
				setStatusCapability(21,1); // results
			}
		}
		catch(Exception e)
		{
			ExceptionReport("query", e);
			ExceptionReport("Testing HTML format in Query ", e);
		}
		
		r.returnFormat = OGC_CatalogService.CG_MessageFormat.TXT; 
		try
		{
			resp = m_Catalog.query(r);
			if(isCorrect(resp.status))
			{
				testFormatQuery(r,resp);
				setStatusCapability(21,1); // results
			}
		}
		catch(Exception e)
		{
			ExceptionReport("query", e);
			ExceptionReport("Testing TXT Format in Query ", e);
		}

		r.returnFormat = OGC_CatalogService.CG_MessageFormat.NV; 
		r.sortField = new OGC_CatalogService.CG_SortField[1];  	
		r.sortField[0] = new OGC_CatalogService.CG_SortField ("title", OGC_CatalogService.CG_SortOrder.ascending);
		try
		{			
			resp = m_Catalog.query(r);

			if(isCorrect(resp.status))
			{
				testFormatQuery(r,resp);		
				localHits = numberHits(resp);//in distributed scope this number has to be greater
				
				diagnosticArea.append("Local query has returned " + resp.hits + " hits of " + correctHits +" expected\n");
				if (localHits != correctHits)								
				{
					setStatusCapability(25,0);//scope=local				
					setStatusCapability(21,0);//results				
				}
				else
				{
					setStatusCapability(25,1);//scope=local
					setStatusCapability(21,1);//results				
					setStatusCapability(12,1);//OGC_COMMON
				}
				// Testing Order
				if (getStatusCapability(17)==1) // if NV format is working
				{
					if(testOrder("ascending"))
						setStatusCapability(19,1); // ascerding order
					else
						setStatusCapability(19,0);
				}
				else{}
			}
			else
			{
				setStatusCapability(21,0);//results
				setStatusCapability(12,0);//language OGC_COMMON
				setStatusCapability(25,0);//scope=local				
			}
		} 
		catch (Exception e)	{
			ExceptionReport("Testing NV Format in Query ", e);
		}
		
		// Testing the GeographicBoundingBox query -> correctHitsCoord = 2				
		thequery ="intersects(GeographicBoundingBox, envelope(-24.17,40.71,71.26,27.63))"; //west, east, north, south
		r.queryExpression = new OGC_CatalogService.CG_QueryExpression(thequery,"",language);
		try
		{
			resp = m_Catalog.query(r);

			if(isCorrect(resp.status))
			{
				if (numberHits(resp)!= correctHitsCoord)								
				{
					setStatusCapability(27,0);//Query GeographicBoundingBox
				}
				else
					setStatusCapability(27,1);
			}
			else
			{
				setStatusCapability(27,0);
			}
		} 
		catch (Exception e)	{
			ExceptionReport("Testing GeographicBoundingBox Query ", e);
		}		

		// Testing the invalidate query
		thequery = "abstract lkdd 'river%";
		r.queryExpression = new OGC_CatalogService.CG_QueryExpression(thequery,"",language);
		try
		{
			resp = m_Catalog.query(r);

			if(isCorrect(resp.status))
				setStatusCapability(28,0); //If the server send a rigth status is not working
			else
				setStatusCapability(28,1);
		} 
		catch (Exception e)	{
			setStatusCapability(28,1); // Exception because is a invalid query
			ExceptionReport("Testing invalid Query ", e);
		}		

		//query with descending order, with the default query
		thequery =getQuery();
		r.queryExpression = new OGC_CatalogService.CG_QueryExpression(thequery,"",language);		
		r.sortField[0] = new OGC_CatalogService.CG_SortField ("title", OGC_CatalogService.CG_SortOrder.descending);
		removeVector(); //remove all the elements for testing now the descending mode. 
		try
		{
			resp = m_Catalog.query(r);
			if(isCorrect(resp.status))
			{
				// Testing Order
				if (getStatusCapability(17)==1)
				{
					OGIS.NVPair[] Meta = OGIS.NVPairSeqHelper.read(resp.retrievedData.payload.create_input_stream());
						
					// Add all hits
					for (int i = 0; i < Meta.length; i++)
					{
						getAttribute(Meta[i], 0,"Title"); // for testing the order
					}

					if(testOrder("descending"))
						setStatusCapability(20,1);
					else
						setStatusCapability(20,0);
				}
				else{}
			}
			else
				setStatusCapability(20,0); 			
		} 
		catch (Exception e)	{
			//setStatusCapability(20,0);
			ExceptionReport("Testing descending order in Query ", e);
		}

		r.sortField = new OGC_CatalogService.CG_SortField[0];  	
		r.queryScope = OGC_CatalogService.CG_QueryScope.distributed;
		try
		{
			resp = m_Catalog.query(r);

			if(isCorrect(resp.status))
			{
				diagnosticArea.append("Distributed query has returned " + resp.hits + " hits of "+(correctHits*2)+" expected\n");
				if (numberHits(resp)==(correctHits*2)) // 2 distributed servers for testing				
					setStatusCapability(26,1); // scope distributed				
				else
					setStatusCapability(26,0); // scope distributed   
			}
			else
			{
				setStatusCapability(26,0); // scope distributed
			}
		} 
		catch (Exception e)	{
			ExceptionReport("Testing distributed Query ", e);}

		//Testing funcionality of differents languages.
		
		language = OGC_CatalogService.CG_QueryLanguage.Z3950_TypeOne;
		thequery = "Europe[1,4]";
		r.queryExpression = new OGC_CatalogService.CG_QueryExpression(thequery,"",language);
		r.queryScope = OGC_CatalogService.CG_QueryScope.local;
		
		try
		{
			resp = m_Catalog.query(r);
			if(isCorrect(resp.status))			
			{
				if (numberHits(resp)==correctHits)
					setStatusCapability(13,1);//language z39.50			
				else
					setStatusCapability(13,0);//language z39.50			
			}
			else			
				setStatusCapability(13,0);			
		}
		catch(Exception e)
		{			
			ExceptionReport("Testing z39.50 language in Query ", e);
		}

		// now we test the resultType: resuts, hits, validate, resultSetD
		language = getLanguage(); // OGC_Common Language again
		thequery =getQuery();
		r.returnFormat = OGC_CatalogService.CG_MessageFormat.TXT; 
		r.queryExpression = new OGC_CatalogService.CG_QueryExpression(thequery,"",language);		
		r.resultType = OGC_CatalogService.CG_ResultType.validate;
		r.catalogType = OGC_CatalogService.CG_CatalogEntryType.collection;
		try
		{
			resp = m_Catalog.query(r);
			empty = noDataReturned(resp);
			if(isCorrect(resp.status) && (empty))
			{				
				setStatusCapability(22,1); //validate
			}
			else
			{
				setStatusCapability(22,0);
			}
		} 
		catch (Exception e)
		{
			setStatusCapability(22,0);
			ExceptionReport("Testing resultType=validate", e);
		}

		r.resultType = OGC_CatalogService.CG_ResultType.hits;
		try
		{
			resp = m_Catalog.query(r);
			//Testing that retrieveData == empty
			empty = noDataReturned(resp);
			if(isCorrect(resp.status) && (empty))
			{
				setStatusCapability(23,1); //hits
			}
			else
			{
				setStatusCapability(23,0);
			}
		} 
		catch (Exception e)
		{
			setStatusCapability(23,0);
			ExceptionReport("Testing resultType=hits", e);
		}

		r.resultType = OGC_CatalogService.CG_ResultType.resultSetID;
		try
		{
			resp = m_Catalog.query(r);			
			OGC_CatalogService.CG_CollectionName resultSetID = new OGC_CatalogService.CG_CollectionName();
			resultSetID = resp.resultSetID;

			empty = noDataReturned(resp);
			if(isCorrect(resp.status) && empty)
			{
				setStatusCapability(24,1); //resultSetID
			}
			else
			{				
				setStatusCapability(24,0);
			}
			
			if (getStatusCapability(9)==1)
			{
				diagnosticArea.append("Query general status = success \n");
			}
			else
				diagnosticArea.append("Query general status = failure \n");
			
			present(resultSetID);
		} 
		catch (Exception e)
		{
			setStatusCapability(24,0);
			ExceptionReport("Testing resultType=resultSetID", e);
		}
}


// -------------------------------------------------------------------
// Method:    query
// -------------------------------------------------------------------
// Purpose:   the query when the user click the submit button
// -------------------------------------------------------------------
void query(String thequery)   	//OGC_CatalogService.CG_CollectionName query(String thequery)
{
		OGC_CatalogService.CG_QueryLanguage language;
		OGC_CatalogService.CG_QueryRequest r= new OGC_CatalogService.CG_QueryRequest();
		r.base = BaseGet();
		language = getLanguage();
		r.queryExpression = new OGC_CatalogService.CG_QueryExpression(thequery,"",language);
		r.resultType = OGC_CatalogService.CG_ResultType.results;
		r.iteratorSize = 100;
		r.cursor = 0;
		r.returnFormat = OGC_CatalogService.CG_MessageFormat.NV; // Name-value pairs
		r.presentation = new OGC_CatalogService.CG_PresentationDescription();
		r.presentation.presentationType(OGC_CatalogService.CG_PredefinedPresentationType.full);
		r.sortField = new OGC_CatalogService.CG_SortField[0];
		r.queryScope = OGC_CatalogService.CG_QueryScope.local;
		r.collectionID = new OGC_CatalogService.CG_CollectionName();
		r.collectionID.collectionID("");
		r.catalogType = OGC_CatalogService.CG_CatalogEntryType.collection;
		r.asynchronous = false;
		r.maxLevel = 2;

		OGC_CatalogService.CG_QueryResponse resp;
		try
		{
			setStatus("Processing query...");			
			resp = m_Catalog.query(r);
			diagnosticArea.append("Query has returned " + resp.hits + " hits. \n");
			
			if(isCorrect(resp.status))
			{	
				try
				{	 
					diagnosticArea.append("Query retrievedData Format: NV \n");
					OGIS.NVPair[] Meta = OGIS.NVPairSeqHelper.read(resp.retrievedData.payload.create_input_stream());
					// Add all hits
					for (int i = 0; i < Meta.length; i++)
						NVAdd(Meta[i], 0);				
				}
				catch ( Exception e ){
					ExceptionReport("Reading data", e);
				}
			}
			showStatus("Query",resp.status);
		}
		catch (Exception e)	{
			ExceptionReport("User query", e);
		}
}


// -------------------------------------------------------------------
// Method:    explainCollection
// -------------------------------------------------------------------
// Purpose:   for retrieving the schema of the DB and test the capabilities
// -------------------------------------------------------------------
void explainCollection()
{
	int excp = 0;
	OGC_CatalogService.CG_ExplainCollectionRequest r = new OGC_CatalogService.CG_ExplainCollectionRequest();
	r.base = BaseGet();

	r.collectionID = new OGC_CatalogService.CG_CollectionName();
	r.collectionID.collectionID("");
	r.returnFormat = OGC_CatalogService.CG_MessageFormat.NV;
	r.attributeCategory = OGC_CatalogService.CG_AttributeCategory.both; // {queriable, presentable, both}

	OGC_CatalogService.CG_ExplainCollectionResponse resp;

	setStatusCapability(1,0);// If no success in none request it will be not supported
	setStatus("Explain Collection");	
	try
	{
		resp = m_Catalog.explainCollection(r);
		
		if (isCorrect(resp.status))
		{
			setStatusCapability(1,1);
			setStatusCapability(7,1);//Format NV			
			try
			{
				System.out.println("Testing collectionID.collectionID :" +resp.collectionID.collectionID());
				setStatusCapability(2,1); // collectionID
			}
			catch(Exception e)
			{
				setStatusCapability(2,0); // collectionID
			}

			// we can not test the dataModel in a consistent way
		}
			
	}
	catch(Exception e)
	{		
		ExceptionReport("testing collectionID in ExplainCollection", e);
		excp++;
	}

		r.collectionID.collectionName("");
		try
		{
			resp = m_Catalog.explainCollection(r);

			if (isCorrect(resp.status))
			{				
				setStatusCapability(1,1);
				try
				{
					System.out.println("Testing collectionID.collectionName :" +resp.collectionID.collectionName());					
					setStatusCapability(3,1);//collectionName
				}
				catch(Exception e)
				{					
					setStatusCapability(3,0);//collectionName
					excp ++;
				}
			}			
		}
		catch(Exception e){
			ExceptionReport("testing collectionName in ExplainCollection", e);
			//setStatusCapability(1,0);
			excp ++;
		}

		r.returnFormat = OGC_CatalogService.CG_MessageFormat.XML;
		r.collectionID.collectionID("");
		try
		{
			resp = m_Catalog.explainCollection(r);
			if (isCorrect(resp.status))
			{				
				setStatusCapability(1,1);
				setStatusCapability(4,1);//Format XML
			}
		}
		catch(Exception e){
			ExceptionReport("testing XML Format in ExplainCollection", e);
			excp ++;
		}

		r.returnFormat = OGC_CatalogService.CG_MessageFormat.HTML;
		try
		{
			resp = m_Catalog.explainCollection(r);
			if (isCorrect(resp.status))
			{				
				setStatusCapability(1,1);
				setStatusCapability(5,1);//Format HTML
			}
		}
		catch(Exception e){
			ExceptionReport("testing HTML Format in ExplainCollection", e);
			excp ++;
		}
		
		r.returnFormat = OGC_CatalogService.CG_MessageFormat.TXT;
		try
		{
			resp = m_Catalog.explainCollection(r);
			if (isCorrect(resp.status))
			{				
				setStatusCapability(1,1);
				setStatusCapability(6,1);//Format TXT
			}
		}
		catch(Exception e){
			ExceptionReport("testing TXT Format in ExplainCollection", e);
			excp ++;
		}

		diagnosticArea.append(excp +" exceptions occured during the Explain collection. \n");
		if (getStatusCapability(1)==1)
		{
			diagnosticArea.append("Explain collection general status = success \n");	
		}
		else
			diagnosticArea.append("Explain collection general status = failure \n");	
}


// -------------------------------------------------------------------
// Method:    present
// -------------------------------------------------------------------
// Purpose:   present()call with the resultSetID obtained in the query()
//            and test the diferents parameters in the present "struct"
// -------------------------------------------------------------------
void present(OGC_CatalogService.CG_CollectionName resultID)
{	
	int excp = 0; 
	OGC_CatalogService.CG_PresentRequest r = new OGC_CatalogService.CG_PresentRequest();
	r.base = BaseGet();

	r.iteratorSize = 100;
	r.cursor = 0;
	r.presentation = new OGC_CatalogService.CG_PresentationDescription();
	r.presentation.presentationType(OGC_CatalogService.CG_PredefinedPresentationType.full);
	r.resultSetID = resultID;
	r.sortField = new OGC_CatalogService.CG_SortField[0];
	
	OGC_CatalogService.CG_PresentResponse resp;
	
	setStatusCapability(29,0);
	r.returnFormat = OGC_CatalogService.CG_MessageFormat.XML; 
	setStatus("Present");	
	try
	{
		resp = m_Catalog.present(r);
		if(isCorrect(resp.status))
		{
			setStatusCapability(29,1);
			showStatus("Present",resp.status);
			testFormatPresent(r,resp); 			
		}
	}
	catch (Exception e)	{
		ExceptionReport("testing XML Format in present", e);
		excp ++;
	}
	
	r.returnFormat = OGC_CatalogService.CG_MessageFormat.HTML; 
	try
	{
		resp = m_Catalog.present(r);		
		if(isCorrect(resp.status))
		{
			setStatusCapability(29,1);
			testFormatPresent(r,resp); 
		}
	}
	catch (Exception e)	{
		ExceptionReport("testing HTML Format in present", e);
		excp ++;
	}
	r.returnFormat = OGC_CatalogService.CG_MessageFormat.TXT; 
	try
	{
		resp = m_Catalog.present(r);
		if (isCorrect(resp.status))
		{
			setStatusCapability(29,1);
			testFormatPresent(r,resp); 			
		}	
	}
	catch (Exception e)	{
		ExceptionReport("testing TXT Format in present", e);
		excp ++;
	}	

	r.returnFormat = OGC_CatalogService.CG_MessageFormat.NV; // Name-value pairs
	r.sortField = new OGC_CatalogService.CG_SortField[1];  	
	r.sortField[0] = new OGC_CatalogService.CG_SortField ("title", OGC_CatalogService.CG_SortOrder.ascending);	
	try
	{		
		resp = m_Catalog.present(r);
		diagnosticArea.append("Present has returned " + resp.hits + " hits of " +correctHits+" expected\n");

		if(isCorrect(resp.status))
		{
			setStatusCapability(29,1);
			removeVector();
			testFormatPresent(r,resp); 
			// Testing Order			
			if(getStatusCapability(33)==1)
			{
				if(testOrder("ascending"))
					setStatusCapability(35,1);
				else
					setStatusCapability(35,0);
			}
		}				
	}
	catch (Exception e)	{
		ExceptionReport("testing NV Format in present", e);
		excp ++;
	}

	r.sortField[0] = new OGC_CatalogService.CG_SortField ("title", OGC_CatalogService.CG_SortOrder.descending);	
	try
	{		
		resp = m_Catalog.present(r);		
		if(isCorrect(resp.status))
		{
			setStatusCapability(29,1);
			// Testing Order
			removeVector();
			if(getStatusCapability(33)==1)
			{
				OGIS.NVPair[] Meta = OGIS.NVPairSeqHelper.read(resp.retrievedData.payload.create_input_stream());
						
				// Add all hits
				for (int i = 0; i < Meta.length; i++)
				{
						getAttribute(Meta[i], 0,"Title"); // for testing the order
				}
				if(testOrder("descending"))
					setStatusCapability(36,1);
				else
					setStatusCapability(36,0);
			}
		}
	}
	catch (Exception e)	{
		excp ++;
	}

	diagnosticArea.append(excp + " exception occured during Present \n");
	if (getStatusCapability(29)==1)
	{
		diagnosticArea.append("Present general status = success \n");	
	}
	else
		diagnosticArea.append("Present general status = failure \n");	
}


// -------------------------------------------------------------------
// Method:    explainServer
// -------------------------------------------------------------------
// Purpose:   Make explainServer request
// -------------------------------------------------------------------
void explainServer()  
{

	OGC_CatalogService.CG_ExplainServerRequest r= new OGC_CatalogService.CG_ExplainServerRequest();
	OGC_CatalogService.CG_ExplainServerResponse resp;

	r.base = BaseGet();
	r.capabilities = new OGC_CatalogService.CG_CapabilityType[9];
	r.capabilities[0] = OGC_CatalogService.CG_CapabilityType.ctAllSupportedRequest;  //Asking for all the capabilities
	r.capabilities[1] = OGC_CatalogService.CG_CapabilityType.ctDefaults;
	r.capabilities[2] = OGC_CatalogService.CG_CapabilityType.ctDefaultTimeOut;
	r.capabilities[3] = OGC_CatalogService.CG_CapabilityType.ctExplain;
	r.capabilities[4] = OGC_CatalogService.CG_CapabilityType.ctMessaging;
	r.capabilities[5] = OGC_CatalogService.CG_CapabilityType.ctQuery;
	r.capabilities[6] = OGC_CatalogService.CG_CapabilityType.ctSession;
	r.capabilities[7] = OGC_CatalogService.CG_CapabilityType.ctSoftwareInformation;
	r.capabilities[8] = OGC_CatalogService.CG_CapabilityType.ctSupportedCollections;

	setStatus("Explain Server");	
	try
	{ 
		resp = m_Catalog.explainServer(r);

		diagnosticArea.append("Number of capabilities returned by ExplainServer: " + resp.capabilities.length + "\n");
		
		if (resp.capabilities.length >0)//we can not test isCorrect(resp.status)->no status in the explainServer response 
		{	
			setStatusCapability(8,1);
		}
		else
			setStatusCapability(8,0);
	}
	catch (Exception e)
	{
		setStatusCapability(8,0);
		ExceptionReport("seeCapabilities",e);
	}	
}



// -------------------------------------------------------------------
// Method:    Submit
// -------------------------------------------------------------------
// Purpose:   Make a query() call when submit button has been pressed
// -------------------------------------------------------------------
public void Submit()
{
	String s;
	String args[] = new String[1];
	args[0]="";
	boolean succ;
		
	resultsArea.setText("");
	diagnosticArea.setText("");

	setStatus("Processing...");
	if (initSession(args))
	{					
			diagnosticArea.append("Init Session status = success\n");
			s = getQuery();
			query(s);
			succ = terminateSession();
			setStatus("Waiting for a new query ...");
	}
	else
		diagnosticArea.append("Init Session status = failure\n");
}


// -------------------------------------------------------------------
// Method:    getQuery
// -------------------------------------------------------------------
// Purpose:   reads the queryText field 
// -------------------------------------------------------------------
public String getQuery()
{
	return(queryText.getText());
}

public void showCapabilities()
{
	int i;

	for (i=0;i<(38+QueryVector.size());i++)
		addCapability(tableCapabilities.name[i],tableCapabilities.available[i]);
}
	

// -------------------------------------------------------------------
// Method:    setStatus
// -------------------------------------------------------------------
// Purpose:   Show in the StatusText field which is the status indicated 
//            by the String
// -------------------------------------------------------------------
public void setStatus(String s)
{
	statusText.setText(s);
}


// -------------------------------------------------------------------
// Method:    queryList
// -------------------------------------------------------------------
// Purpose:   Section which checks list of queries from file query.txt
// -------------------------------------------------------------------
public void queryList()
{
	setStatus("Queries");	
	//queryText.setText((String) QueryVector.elementAt(0));
	OGC_CatalogService.CG_QueryRequest r2= new OGC_CatalogService.CG_QueryRequest();
	r2.base = BaseGet();
	int localHits;
	boolean empty;
	for (int i=0; i<QueryVector.size(); ++i)
	{
		try
		{
			String StringLanguage = (String)QueryLanguageVector.elementAt(i);
			OGC_CatalogService.CG_QueryLanguage language2;
			if (StringLanguage.compareTo("OGC_Common") == 0)
				language2 = OGC_CatalogService.CG_QueryLanguage.OGC_Common;
			else if (StringLanguage.compareTo("Z3950_TypeOne") == 0)
				language2 = OGC_CatalogService.CG_QueryLanguage.Z3950_TypeOne;
			else throw new Exception("Bad query language " + StringLanguage);	

			r2.queryExpression = new OGC_CatalogService.CG_QueryExpression((String)QueryVector.elementAt(i),"",language2);
			r2.iteratorSize = 100;
			r2.cursor = 0;
			r2.asynchronous = false;
			r2.maxLevel = 2;
			r2.queryScope = OGC_CatalogService.CG_QueryScope.local;		
			r2.presentation = new OGC_CatalogService.CG_PresentationDescription();
			r2.presentation.presentationType(OGC_CatalogService.CG_PredefinedPresentationType.full);
			r2.sortField = new OGC_CatalogService.CG_SortField[0];  	
			r2.collectionID = new OGC_CatalogService.CG_CollectionName();	
			r2.collectionID.collectionID("");
			r2.catalogType = OGC_CatalogService.CG_CatalogEntryType.collection;		
			r2.resultType = OGC_CatalogService.CG_ResultType.results;
			r2.returnFormat = OGC_CatalogService.CG_MessageFormat.NV;
			OGC_CatalogService.CG_QueryResponse resp2;

			resp2 = m_Catalog.query(r2);
			localHits = numberHits(resp2);
			int correctHits = Integer.parseInt((String)NumberOfHits.elementAt(i));
			try
			{
				if (localHits != correctHits)
				{
					setStatusCapability(38+i,0);
				}
				else
					setStatusCapability(38+i,1);
			}
			catch(Exception e)
			{
				setStatusCapability(38+i,0);
			}
		}
		catch(Exception e)
		{
			setStatusCapability(38+i,0);
			e.printStackTrace();
		}
	}
}

// -------------------------------------------------------------------
// Method:    main
// -------------------------------------------------------------------
// Purpose:   Main class starting up the testsuite
// -------------------------------------------------------------------
static public void main(String args[])
{
		String theQuery = new String();
		boolean succ;
		OGC_CatalogService.CG_CollectionName resultID;

		try 
		{
		    // Add the following code if you want the Look and Feel of the native system.
		    /*try {UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());} 
			   catch (Exception e) {}*/

			catalogCertification C = new catalogCertification();
			C.setVisible(true);

			// --- Certification code ---
	
			if (C.initSession(args))
			{	
				C.diagnosticArea.append("Init Session status = success\n");
				C.setStatusCapability(0,1);
				C.explainCollection();	
				C.explainServer();
				theQuery = C.getQuery();
				C.queryTest(theQuery);
				C.queryList();
				if (C.terminateSession())
				{
					C.setStatusCapability(37,1);
				}
				else
				{
					C.setStatusCapability(37,0);
				}
			}
			else
			{
				C.diagnosticArea.append("Init Session status = failure\n");
				C.setStatusCapability(0,0);
			}

			C.showCapabilities();

			//Query enable, Certification process finished
			submitButton.setEnabled(true);
			C.setStatus("Waiting for a new query...");															
		} 
		catch (Throwable t) 
		{
			t.printStackTrace();
			//Ensure the application exits with an error condition.
			System.exit(1);
		}
}//---------------------------End main()

	// Used by addNotify
	boolean frameSizeAdjusted = false;
    /**
     * Notifies this component that it has been added to a container
     * This method should be called by <code>Container.add</code>, and 
     * not by user code directly.
     * Overridden here to adjust the size of the frame if needed.
     * @see java.awt.Container#removeNotify
     */

	public void addNotify()
	{
		// Record the size of the window prior to calling parents addNotify.
		Dimension size = getSize();		

		super.addNotify();
		
		if (frameSizeAdjusted)
			return;
		frameSizeAdjusted = true;
		
		// Adjust size of frame according to the insets and menu bar
		JMenuBar menuBar = getRootPane().getJMenuBar();
		int menuBarHeight = 0;
		if (menuBar != null)
		    menuBarHeight = menuBar.getPreferredSize().height;
		Insets insets = getInsets();
		setSize(insets.left + insets.right + size.width, insets.top + insets.bottom + size.height + menuBarHeight);
	}	

	void exitApplication()
	{
		try 
		{
	    	int reply = JOptionPane.showConfirmDialog(this,"Do you really want to exit?","OGC Catalog Certification." , 
	    	                                          JOptionPane.YES_NO_OPTION, 
	    	                                          JOptionPane.QUESTION_MESSAGE);
			if (reply == JOptionPane.YES_OPTION)
			{
		    	this.setVisible(false);    // hide the Frame
		    	this.dispose();            // free the system resources
		    	System.exit(0);            // close the application
			}
		}
		catch (Exception e) {}
	}

	class SymWindow extends WindowAdapter
	{
		public void windowClosing(WindowEvent event)
		{
			Object object = event.getSource();		
			if (object == catalogCertification.this)
				Client_windowClosing(event);
		}
	}

	void Client_windowClosing(java.awt.event.WindowEvent event)
	{
		// to do: code goes here.			 
		Client_windowClosing_Interaction1(event);
	}

	void Client_windowClosing_Interaction1(WindowEvent event) 
	{
		try 
		{
			this.exitApplication();
		} 
		catch (Exception e) {}
	}
	
	class submitQuery implements ActionListener
	{
		public void actionPerformed(ActionEvent event)
		{			
			 Object object = event.getSource(); 
			 Submit();
		}
	}
}

